/*

  SAMPTAIL.C	Sample .MEX file corresponding to SAMPTAIL.M

  function samp=samptail(mu,v);
  
  Sample from the positive tail of a Gaussian, mu has to be negative. 
  Will sample from the distribution of x given that x>0. 

*/

#include <math.h>
#include <stdlib.h>
#include "mex.h"

/* Input Arguments */

#define	MU_IN	prhs[0]
#define	V_IN	prhs[1]

/* Output Arguments */

#define	SAMP_OUT	plhs[0]

#define	max(A, B)	((A) > (B) ? (A) : (B))
#define	min(A, B)	((A) < (B) ? (A) : (B))
#define pi 3.14159265

static void samptail(
	double	*samp,
	double	*mu,
	double	*v
	)
{
  double tiny,lambda, y, k, accept;
  double pexp0,pgauss0,pexpy, pgaussy;
  int done=0;

  tiny=exp(-600);

  if (mu[0]>0)
    {
      printf("samptail: mu must be negative\n");
      return;
    }

  /* form an exponential distribution bound by looking at the value
     and derivative at zero */

  lambda=-v[0]/(mu[0]+tiny);
  pexp0=1/(lambda+tiny);   /* prob density of exponential at 0 */
  
  /* prob density of Gaussian at 0 */

  k=1/sqrt(2*pi*v[0]);
  pgauss0=k*exp(-mu[0]*mu[0]/(2*v[0]));   
					      
  while (!done) {

    /* sample from the exp dist */
    y=-lambda*log(drand48());
    
    /* reject according to relative probabilities */
	
    pgaussy=k*exp(-(y-mu[0])*(y-mu[0])/(2*v[0]));
    pexpy=1/lambda*exp(-y/lambda);
    
    if (pexpy*pgauss0==0)
      accept=1;
    else
      accept=pgaussy*pexp0/(pexpy*pgauss0);
    
    if (drand48()<accept)
      done=1;
  }
  *samp=y;
}

#ifdef __STDC__
void mexFunction(
	int		nlhs,
	Matrix	*plhs[],
	int		nrhs,
	Matrix	*prhs[]
	)
#else
mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
#endif
{
	double	*samp, *mu, *v;
	unsigned int	m,n;

	/* Check for proper number of arguments */

	if (nrhs != 2) {
		mexErrMsgTxt("SAMPTAIL requires two input arguments.");
	} else if (nlhs > 1) {
		mexErrMsgTxt("SAMPTAIL requires one output argument.");
	}

	/* Check the dimensions of Y.  Y can be 4 X 1 or 1 X 4. */

	m = mxGetM(MU_IN);
	n = mxGetN(MU_IN);
	if (!mxIsNumeric(MU_IN) || mxIsComplex(MU_IN) || 
		!mxIsFull(MU_IN)  || !mxIsDouble(MU_IN) ||
		(max(m,n) != 1) || (min(m,n) != 1)) {
		mexErrMsgTxt("SAMPTAIL requires that mu be a 1 x 1 vector.");
	}

	/* Create a matrix for the return argument */

	SAMP_OUT = mxCreateFull(1, 1, REAL);

	/* Assign pointers to the various parameters */

	samp = mxGetPr(SAMP_OUT);
	mu = mxGetPr(MU_IN);
	v = mxGetPr(V_IN);

	/* Do the actual computations in a subroutine */
	samptail(samp,mu,v);
	return;
}


